#!/usr/bin/env node
"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chokidar_1 = __importDefault(require("chokidar"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const minimist_1 = __importDefault(require("minimist"));
const introspectMachine_1 = require("./introspectMachine");
const extractMachines_1 = require("./extractMachines");
const printToFile_1 = require("./printToFile");
const _a = minimist_1.default(process.argv.slice(2)), { _: patterns } = _a, objectArgs = __rest(_a, ["_"]);
const onlyOnce = objectArgs.once;
if (patterns.length === 0) {
    console.log('You must pass at least one glob, for instance "**/src/**.machine.ts"');
    process.exit(1);
}
const typedSuffix = /\.typed\.(js|ts|tsx|jsx)$/;
const tsExtension = /\.(ts|tsx|js|jsx)$/;
function isValidFile(filePath) {
    return !typedSuffix.test(filePath) && tsExtension.test(filePath);
}
const fileCache = {};
printToFile_1.printJsFiles();
printToFile_1.printToFile(fileCache, objectArgs.outDir);
if (!onlyOnce) {
    console.clear();
}
const watcher = chokidar_1.default.watch(patterns, {
    persistent: !onlyOnce,
});
watcher.on('error', (err) => {
    console.error(err);
    process.exit(1);
});
const toRelative = (filePath) => path_1.default.relative(process.cwd(), filePath);
watcher.on('all', async (eventName, filePath) => {
    if (!isValidFile(filePath)) {
        return;
    }
    const relativePath = toRelative(filePath).gray;
    if (eventName === 'add') {
        console.log(`Scanning File: `.cyan.bold + ` ${relativePath}`);
        try {
            await addToCache(filePath);
        }
        catch (err) {
            console.log(`Could not complete due to errors in ${relativePath}`.red.bold);
            console.log(err);
        }
    }
    if (eventName === 'change') {
        console.log(`File Changed: `.yellow.bold + ` ${relativePath}`);
        try {
            await addToCache(filePath);
        }
        catch (err) {
            console.log(`Could not complete due to errors in ${relativePath}`.red.bold);
            console.log(err);
        }
    }
    if (eventName === 'unlink') {
        console.log(`File Deleted: `.red.bold + ` ${relativePath}`);
        removeFromCache(filePath);
    }
    printToFile_1.printToFile(fileCache, objectArgs.outDir);
});
process.on('exit', () => {
    if (onlyOnce) {
        // little trick because `ready` doesn't work well to know the inital run is complete
        console.log('Completed!'.green.bold);
    }
});
watcher.on('ready', async () => {
    if (!onlyOnce) {
        patterns.forEach((pattern) => {
            console.log(`Watching for file changes in: `.cyan.bold + pattern);
        });
    }
});
async function addToCache(filePath) {
    let code = '';
    try {
        code = fs_1.default.readFileSync(filePath, 'utf8');
    }
    catch (e) { }
    if (!code) {
        throw new Error(`Could not read from path ${filePath}`);
    }
    if (!code.includes('@xstate/compiled')) {
        return;
    }
    const machines = await extractMachines_1.extractMachines(filePath);
    if (machines.length === 0) {
        return;
    }
    const { machine, id } = machines[0];
    fileCache[filePath] = Object.assign(Object.assign({}, introspectMachine_1.introspectMachine(machine)), { id });
}
function removeFromCache(filePath) {
    delete fileCache[filePath];
}
