"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRootNode = exports.getMatchesStates = exports.getTransitionsFromNode = void 0;
const utils_1 = require("xstate/lib/utils");
exports.getTransitionsFromNode = (node) => {
    const transitions = new Set();
    if (node.parent) {
        Object.keys(node.parent.states).forEach((key) => transitions.add(key));
        Object.values(node.parent.states).forEach((siblingNode) => {
            exports.getMatchesStates(siblingNode).forEach((key) => {
                var _a;
                if (key === siblingNode.path.join('.')) {
                    return;
                }
                let relativeKey = key;
                if ((((_a = node.parent) === null || _a === void 0 ? void 0 : _a.path.length) || 0) > 0) {
                    relativeKey = relativeKey.replace(new RegExp(`^${node.parent.path.join('.')}\.`), '');
                }
                transitions.add(relativeKey);
            });
        });
    }
    Object.values(node.states).map((childNode) => {
        exports.getMatchesStates(childNode).map((key) => {
            var _a;
            let relativeKey = key;
            if ((((_a = childNode.parent) === null || _a === void 0 ? void 0 : _a.path.length) || 0) > 0) {
                relativeKey = relativeKey.replace(new RegExp(`^${childNode.parent.path.join('.')}\.`), '');
            }
            transitions.add(`.${relativeKey}`);
            transitions.add(`${relativeKey}`);
        });
    });
    const rootNode = exports.getRootNode(node);
    const nodesWithId = rootNode.stateIds
        .filter((id) => !/(\.|\(machine\))/.test(id))
        .map((id) => rootNode.getStateNodeById(id));
    nodesWithId.forEach((idNode) => {
        exports.getMatchesStates(idNode).forEach((match) => {
            if (idNode.id === rootNode.id) {
                transitions.add(`#${idNode.id}.${match}`);
                return;
            }
            transitions.add(match.replace(new RegExp(`^${idNode.path.join('.')}`), `#${idNode.id}`));
        });
    });
    utils_1.toStatePaths(utils_1.pathToStateValue(node.path)).forEach((path) => {
        if (path.length > 1) {
            transitions.delete(path.join('.'));
        }
    });
    return Array.from(transitions);
};
exports.getMatchesStates = (machine) => {
    const allStateNodes = machine.stateIds.map((id) => machine.getStateNodeById(id));
    const states = allStateNodes.reduce((arr, node) => {
        return [
            ...arr,
            ...utils_1.toStatePaths(utils_1.pathToStateValue(node.path)).map((path) => path.join('.')),
        ];
    }, []);
    return states;
};
exports.getRootNode = (node) => {
    if (!node.parent) {
        return node;
    }
    return exports.getRootNode(node.parent);
};
