"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isNodeModulePath = exports.explodeAbsolutePath = exports.findLastIndex = void 0;
const path_1 = __importDefault(require("path"));
exports.findLastIndex = (arr, predicate) => {
    for (let index = arr.length - 1; index >= 0; index--) {
        if (predicate(arr[index])) {
            return index;
        }
    }
    return -1;
};
exports.explodeAbsolutePath = (filePath) => {
    if (!path_1.default.isAbsolute(filePath)) {
        throw new Error(`Received file path is not an absolute one: ${filePath}`);
    }
    const parsed = path_1.default.parse(filePath);
    return [...parsed.dir.slice(parsed.root.length).split(path_1.default.sep), parsed.base];
};
exports.isNodeModulePath = (moduleName, absoluteFilePath) => {
    if (!moduleName.length) {
        throw new Error('Received `moduleName` has to be non-empty.');
    }
    const explodedPath = exports.explodeAbsolutePath(absoluteFilePath);
    const nodeModulesIndex = exports.findLastIndex(explodedPath, (segment) => segment === 'node_modules');
    if (nodeModulesIndex === -1) {
        return false;
    }
    if (moduleName[0] === '@') {
        return (`${explodedPath[nodeModulesIndex + 1]}/${explodedPath[nodeModulesIndex + 2]}` === moduleName);
    }
    return explodedPath[nodeModulesIndex + 1] === moduleName;
};
